//
//  AppDelegate.swift
//  VoltageOutput
//
//  Created by Phidgets on 2018-02-21.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import Cocoa
import Phidget22Swift

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    let ch = VoltageOutput()
    @IBOutlet weak var phidgetInfoBox: PhidgetInfoBox!
    
    //GUI
    @IBOutlet weak var voltageOutputBox: NSBox!
    @IBOutlet weak var voltageSlider: NSSlider!
    @IBOutlet weak var minVoltageLabel: NSTextField!
    @IBOutlet weak var maxVoltageLabel: NSTextField!
    @IBOutlet weak var voltageLabel: NSTextField!
    @IBOutlet weak var outputEnabledBox: NSBox!
    @IBOutlet weak var enableButton: NSButton!
    @IBOutlet weak var rangeBox: NSBox!
    @IBOutlet weak var rangeButton: NSButton!
    
    
    //Error Window
    @IBOutlet weak var errorEventWindow: NSPanel!
    @IBOutlet var errorEventLog: NSTextView!
    @IBOutlet weak var errorEventCount: NSTextField!
    var errorCounter = 0

    //MARK: View events
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        do {
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
            
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            
            /*
             * Please review the Phidget22 channel matching documentation for details on the device
             * and class architecture of Phidget22, and how channels are matched to device features.
             */
            
            /*
             * Specifies the serial number of the device to attach to.
             * For VINT devices, this is the hub serial number.
             *
             * The default is any device.
             */
            // try ch.setDeviceSerialNumber(<YOUR DEVICE SERIAL NUMBER>)
            
            /*
             * For VINT devices, this specifies the port the VINT device must be plugged into.
             *
             * The default is any port.
             */
            // try ch.setHubPort(0)
            
            /*
             * Specifies which channel to attach to.  It is important that the channel of
             * the device is the same class as the channel that is being opened.
             *
             * The default is any channel.
             */
            // try ch.setChannel(0)
            
            /*
             * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
             * In a normal environment this can be done automatically by enabling server discovery, which
             * will cause the client to discovery and connect to available servers.
             *
             * To force the channel to only match a network Phidget, set remote to 1.
             */
            
            // try Net.enableServerDiscovery(serverType: .deviceRemote)
            // try ch.setIsRemote(true)
            
            /*
             openCmdLine allows command line arguments to be passed to this program in order to
             configure channel matching.
             If you have modified the code and added your own channel matching, comment out
             the openCmdLine call, and simply replace it with the following line:
             */
            //try ch.open()
            
            
            try phidgetInfoBox.openCmdLine(phid: ch as Phidget)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func applicationShouldTerminateAfterLastWindowClosed(_ sender: NSApplication) -> Bool {
        return true
    }
    
    func applicationWillTerminate(_ aNotification: Notification) {
        //Safely close example
        do{
            print("Closing")
            ch.attach.removeAllHandlers()
            ch.detach.removeAllHandlers()
            ch.error.removeAllHandlers()
            try ch.close()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event handlers
    func attach_handler(sender: Phidget){
        do{
            let attachedDevice = sender as! VoltageOutput
            let deviceID = try attachedDevice.getDeviceID()

            var enabled:Bool?
            var range:VoltageOutputRange?
            
            if(deviceID == DeviceID.PN_1002 || deviceID == DeviceID.PN_1002){
                enabled = try attachedDevice.getEnabled()
            }
            else{
                range = try attachedDevice.getVoltageOutputRange()
            }
            
            DispatchQueue.main.async {
                self.phidgetInfoBox.fillPhidgetInfo(device: sender)
                self.voltageOutputBox.isHidden = false
                
                self.updateVoltageSlider()
                
                if(deviceID == DeviceID.PN_1002 || deviceID == DeviceID.PN_1002){
                    self.outputEnabledBox.isHidden = false
                    self.enableButton.title = enabled! ? "Disable" : "Enable"
                }
                else{
                    self.rangeBox.isHidden = false
                    self.rangeButton.title = range == VoltageOutputRange.volts_5 ? "Switch to 10V" : "Switch to 5V"
                }
                
                //Adjusting window height
                var frame = self.window.frame
                frame.size.height = self.window.maxSize.height
                self.window.setFrame(frame, display: true, animate: false)
                self.window.center()
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        DispatchQueue.main.async {
            self.phidgetInfoBox.fillPhidgetInfo(device: nil)
            self.rangeBox.isHidden = true
            self.outputEnabledBox.isHidden = true
            self.voltageOutputBox.isHidden = true
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        outputError(errorDescription: data.description, errorCode: data.code.rawValue)
    }
    
    //MARK: GUI Controls
    @IBAction func setOutputVoltage(_ sender: Any) {
        let slider = sender as! NSSlider
        do{
            try ch.setVoltage(slider.doubleValue)
            voltageLabel.stringValue = String(format:"%.2f V", slider.doubleValue)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setEnabled(_ sender: Any) {
        do{
            let button = sender as! NSButton
            let enabled = try ch.getEnabled()
            try ch.setEnabled(!enabled)
            if(enabled){
                button.title = "Enable"
            }
            else{
                button.title = "Disable"
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setRange(_ sender: Any) {
        do{
            let button = sender as! NSButton
            let range = try ch.getVoltageOutputRange()
            if(range == VoltageOutputRange.volts_5){
                try ch.setVoltageOutputRange(VoltageOutputRange.volts_10)
                button.title = "Switch to 5V"
            }
            else{
                try ch.setVoltageOutputRange(VoltageOutputRange.volts_5)
                button.title = "Switch to 10V"
            }
            self.updateVoltageSlider()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Misc
    func updateVoltageSlider(){
        do{
            let minVoltage = try ch.getMinVoltage()
            let maxVoltage = try ch.getMaxVoltage()
            let voltage = try ch.getVoltage()
            DispatchQueue.main.async {
                self.voltageSlider.minValue = minVoltage
                self.voltageSlider.maxValue = maxVoltage
                self.voltageSlider.doubleValue = voltage
                self.minVoltageLabel.stringValue = String(format:"%.2f", minVoltage)
                self.maxVoltageLabel.stringValue = String(format:"%.2f", maxVoltage)
                self.voltageLabel.stringValue = String(format:"%.2f", voltage)
            }
        }catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Error
    func outputError(errorDescription:String, errorCode:UInt32){
        DispatchQueue.main.async {
            self.errorCounter += 1
            let outputString = NSAttributedString(string: String(format:"Error %d: %@\n", errorCode, errorDescription))
            self.errorEventLog.textStorage?.beginEditing()
            self.errorEventLog.textStorage?.append(outputString)
            self.errorEventLog.textStorage?.endEditing()
            self.errorEventCount.stringValue = "\(self.errorCounter)"
            if(!self.errorEventWindow.isVisible){
                self.errorEventWindow.setIsVisible(true)
                self.errorEventWindow.center()
            }
        }
        
    }
    
    @IBAction func clearError(_ sender: Any) {
        errorEventLog.textStorage?.setAttributedString(NSAttributedString(string: ""))
        errorCounter = 0
        errorEventCount.stringValue = "0"
    }
}

